@@ -112,8 +112,9 @@ FlaggedCharsWord flag_similar_characters_word(const PM_Vec& PM, [[maybe_unused]]
     uint64_t BoundMask = bit_mask_lsb<uint64_t>(Bound + 1);
 
     int64_t j = 0;
-    for (; j < std::min(static_cast<int64_t>(Bound), static_cast<int64_t>(T.size())); ++j) {
-        uint64_t PM_j = PM.get(0, T[j]) & BoundMask & (~flagged.P_flag);
+    auto T_iter = T.begin();
+    for (; j < std::min(static_cast<int64_t>(Bound), static_cast<int64_t>(T.size())); ++j,++T_iter) {
+        uint64_t PM_j = PM.get(0, *T_iter) & BoundMask & (~flagged.P_flag);
 
         flagged.P_flag |= blsi(PM_j);
         flagged.T_flag |= static_cast<uint64_t>(PM_j != 0) << j;
@@ -121,8 +122,8 @@ FlaggedCharsWord flag_similar_characters_word(const PM_Vec& PM, [[maybe_unused]]
         BoundMask = (BoundMask << 1) | 1;
     }
 
-    for (; j < T.size(); ++j) {
-        uint64_t PM_j = PM.get(0, T[j]) & BoundMask & (~flagged.P_flag);
+    for (; j < T.size(); ++j,++T_iter) {
+        uint64_t PM_j = PM.get(0, *T_iter) & BoundMask & (~flagged.P_flag);
 
         flagged.P_flag |= blsi(PM_j);
         flagged.T_flag |= static_cast<uint64_t>(PM_j != 0) << j;
@@ -231,8 +232,9 @@ static inline FlaggedCharsMultiword flag_similar_characters_block(const BlockPat
     BoundMask.last_mask = (1ull << (start_range % 64)) - 1;
     BoundMask.first_mask = ~UINT64_C(0);
 
-    for (int64_t j = 0; j < T.size(); ++j) {
-        flag_similar_characters_step(PM, T[j], flagged, static_cast<size_t>(j), BoundMask);
+    auto T_iter = T.begin();
+    for (int64_t j = 0; j < T.size(); ++j,++T_iter) {
+        flag_similar_characters_step(PM, *T_iter, flagged, static_cast<size_t>(j), BoundMask);
 
         if (j + Bound + 1 < P.size()) {
             BoundMask.last_mask = (BoundMask.last_mask << 1) | 1;
@@ -312,9 +314,25 @@ static inline size_t count_transpositions_block(const BlockPatternMatchVector& P
     return Transpositions;
 }
 
+// todo cleanup the split between jaro_bounds
+/**
+ * @brief find bounds
+ */
+static inline int64_t jaro_bounds(int64_t P_len, int64_t T_len)
+{
+    /* since jaro uses a sliding window some parts of T/P might never be in
+     * range an can be removed ahead of time
+     */
+    int64_t Bound = (T_len > P_len) ? T_len : P_len;
+    Bound /= 2;
+    if(Bound > 0)
+        Bound--;
+
+    return Bound;
+}
+
 /**
  * @brief find bounds and skip out of bound parts of the sequences
- *
  */
 template <typename InputIt1, typename InputIt2>
 int64_t jaro_bounds(Range<InputIt1>& P, Range<InputIt2>& T)
@@ -322,6 +340,10 @@ int64_t jaro_bounds(Range<InputIt1>& P, Range<InputIt2>& T)
     int64_t P_len = P.size();
     int64_t T_len = T.size();
 
+    // this is currently an early exit condition
+    // if this is changed handle this below, so Bound is never below 0
+    assert(P_len != 0 || T_len != 0);
+
     /* since jaro uses a sliding window some parts of T/P might never be in
      * range an can be removed ahead of time
      */
@@ -350,7 +372,7 @@ double jaro_similarity(Range<InputIt1> P, Range<InputIt2> T, double score_cutoff
     /* filter out based on the length difference between the two strings */
     if (!jaro_length_filter(P_len, T_len, score_cutoff)) return 0.0;
 
-    if (P_len == 1 && T_len == 1) return static_cast<double>(P[0] == T[0]);
+    if (P_len == 1 && T_len == 1) return static_cast<double>(P.front() == T.front());
 
     int64_t Bound = jaro_bounds(P, T);
 
@@ -432,6 +454,144 @@ double jaro_similarity(const BlockPatternMatchVector& PM, Range<InputIt1> P, Ran
     return (Sim >= score_cutoff) ? Sim : 0;
 }
 
+#ifdef RAPIDFUZZ_SIMD
+template <typename VecType, typename InputIt, int _lto_hack = RAPIDFUZZ_LTO_HACK>
+void jaro_similarity_simd(Range<double*> scores, const detail::BlockPatternMatchVector& block,
+                          const std::vector<int64_t>& s1_lengths, Range<InputIt> s2,
+                          double score_cutoff) noexcept
+{
+#    ifdef RAPIDFUZZ_AVX2
+    using namespace simd_avx2;
+#    else
+    using namespace simd_sse2;
+#    endif
+
+    static constexpr size_t vec_width = native_simd<VecType>::size();
+    static constexpr size_t vecs = static_cast<size_t>(native_simd<uint64_t>::size());
+    assert(block.size() % vecs == 0);
+
+    native_simd<VecType> zero(VecType(0));
+    native_simd<VecType> one(1);
+    size_t result_index = 0;
+
+    if (score_cutoff > 1.0) {
+        for (int64_t i = 0; i < static_cast<int64_t>(s1_lengths.size()); i++)
+            scores[i] = 0.0;
+
+        return;
+    }
+
+    if (s2.empty()) {
+        for (size_t i = 0; i < s1_lengths.size(); i++)
+            scores[static_cast<int64_t>(i)] = s1_lengths[i] ? 0.0 : 1.0;
+
+        return;
+    }
+
+    for (size_t cur_vec = 0; cur_vec < block.size(); cur_vec += vecs) {
+        alignas(32) std::array<VecType, vec_width> boundMaskSize_;
+        alignas(32) std::array<VecType, vec_width> boundMask_;
+
+        auto s2_cur = s2;
+
+        int64_t lastRelevantChar = 0;
+        ptrdiff_t maxBound = 0;
+        unroll<int, vec_width>([&](auto i) {
+            int64_t s1_len = s1_lengths[result_index + i];
+            int64_t Bound = jaro_bounds(s1_len, s2_cur.size());
+
+            if (s1_len + Bound > lastRelevantChar) lastRelevantChar = s1_len + Bound;
+
+            if (Bound > maxBound) maxBound = static_cast<ptrdiff_t>(Bound);
+
+            boundMaskSize_[i] = bit_mask_lsb<VecType>(static_cast<int>(2 * Bound));
+            boundMask_[i] = bit_mask_lsb<VecType>(static_cast<int>(Bound + 1));
+        });
+
+        if (s2_cur.size() > lastRelevantChar) s2_cur.remove_suffix(s2_cur.size() - lastRelevantChar);
+
+        native_simd<VecType> boundMaskSize(reinterpret_cast<uint64_t*>(boundMaskSize_.data()));
+        native_simd<VecType> boundMask(reinterpret_cast<uint64_t*>(boundMask_.data()));
+
+        native_simd<VecType> P_flag(VecType(0));
+        native_simd<VecType> T_flag(VecType(0));
+
+        native_simd<VecType> counter(VecType(1));
+
+        // In case s2 is longer than all of the elements in s1_lengths boundMaskSize
+        // might have all bits set and therefor the condition ((boundMask <= boundMaskSize) & one)
+        // would incorrectly always set the first bit to 1.
+        // this is solved by splitting the loop into two parts where after this boundary is reached
+        // the first bit inside boundMask is no longer set
+        int64_t j = 0;
+        for(; j < std::min(maxBound, s2_cur.size()); ++j)
+        {
+            alignas(32) std::array<uint64_t, vecs> stored;
+            unroll<int, vecs>([&](auto i) { stored[i] = block.get(cur_vec + i, s2_cur[j]); });
+            native_simd<VecType> X(stored.data());
+            native_simd<VecType> PM_j = andnot(X & boundMask, P_flag);
+
+            P_flag |= blsi(PM_j);
+            T_flag |= andnot(counter, (PM_j == zero));
+
+            counter = counter << 1;
+            boundMask = (boundMask << 1) | ((boundMask <= boundMaskSize) & one);
+        }
+
+        for(; j < s2_cur.size(); ++j)
+        {
+            alignas(32) std::array<uint64_t, vecs> stored;
+            unroll<int, vecs>([&](auto i) { stored[i] = block.get(cur_vec + i, s2_cur[j]); });
+            native_simd<VecType> X(stored.data());
+            native_simd<VecType> PM_j = andnot(X & boundMask, P_flag);
+
+            P_flag |= blsi(PM_j);
+            T_flag |= andnot(counter, (PM_j == zero));
+
+            counter = counter << 1;
+            boundMask = boundMask << 1;
+        }
+
+        auto counts = popcount(P_flag);
+        alignas(32) std::array<VecType, vec_width> P_flags;
+        P_flag.store(P_flags.data());
+        alignas(32) std::array<VecType, vec_width> T_flags;
+        T_flag.store(T_flags.data());
+        for (size_t i = 0; i < vec_width; ++i) {
+            VecType CommonChars = counts[i];
+            if (!jaro_common_char_filter(s1_lengths[result_index], s2.size(), CommonChars, score_cutoff)) {
+                scores[static_cast<int64_t>(result_index)] = 0.0;
+                result_index++;
+                continue;
+            }
+
+            VecType P_flag_cur = P_flags[i];
+            VecType T_flag_cur = T_flags[i];
+            size_t Transpositions = 0;
+
+            static constexpr size_t vecs_per_word = vec_width / vecs;
+            size_t cur_block = i / vecs_per_word;
+            int64_t offset = static_cast<int64_t>(sizeof(VecType) * 8 * (i % vecs_per_word));
+            while (T_flag_cur) {
+                VecType PatternFlagMask = blsi(P_flag_cur);
+
+                uint64_t PM_j = block.get(cur_block, s2[countr_zero(T_flag_cur)]);
+                Transpositions += !(PM_j & (static_cast<uint64_t>(PatternFlagMask) << offset));
+
+                T_flag_cur = blsr(T_flag_cur);
+                P_flag_cur ^= PatternFlagMask;
+            }
+
+            double Sim =
+                jaro_calculate_similarity(s1_lengths[result_index], s2.size(), CommonChars, Transpositions);
+
+            scores[static_cast<int64_t>(result_index)] = (Sim >= score_cutoff) ? Sim : 0;
+            result_index++;
+        }
+    }
+}
+#endif /* RAPIDFUZZ_SIMD */
+
 class Jaro : public SimilarityBase<Jaro, double, 0, 1> {
     friend SimilarityBase<Jaro, double, 0, 1>;
     friend NormalizedMetricBase<Jaro>;
