@@ -302,7 +302,7 @@ struct MultiLevenshtein : public detail::MultiDistanceBase<MultiLevenshtein<MaxL
 private:
     friend detail::MultiDistanceBase<MultiLevenshtein<MaxLen>, int64_t, 0,
                                      std::numeric_limits<int64_t>::max()>;
-    friend detail::MultiNormalizedMetricBase<MultiLevenshtein<MaxLen>>;
+    friend detail::MultiNormalizedMetricBase<MultiLevenshtein<MaxLen>, int64_t>;
 
     constexpr static size_t get_vec_size()
     {
@@ -416,7 +416,7 @@ private:
     LevenshteinWeightTable weights;
 };
 } /* namespace experimental */
-#endif
+#endif /* RAPIDFUZZ_SIMD */
 
 template <typename CharT1>
 struct CachedLevenshtein : public detail::CachedDistanceBase<CachedLevenshtein<CharT1>, int64_t, 0,
