@@ -88,12 +88,8 @@ int64_t generalized_levenshtein_wagner_fischer(Range<InputIt1> s1, Range<InputIt
  * @brief calculates the maximum possible Levenshtein distance based on
  * string lengths and weights
  */
-template <typename InputIt1, typename InputIt2>
-int64_t levenshtein_maximum(Range<InputIt1> s1, Range<InputIt2> s2, LevenshteinWeightTable weights)
+static inline int64_t levenshtein_maximum(ptrdiff_t len1, ptrdiff_t len2, LevenshteinWeightTable weights)
 {
-    auto len1 = s1.size();
-    auto len2 = s2.size();
-
     int64_t max_dist = len1 * weights.delete_cost + len2 * weights.insert_cost;
 
     if (len1 >= len2)
@@ -166,7 +162,7 @@ int64_t levenshtein_mbleven2018(Range<InputIt1> s1, Range<InputIt2> s2, int64_t
     assert(len1 > 0);
     assert(len2 > 0);
     assert(*s1.begin() != *s2.begin());
-    assert(*(s1.end() - 1) != *(s2.end() - 1));
+    assert(*(--s1.end()) != *(--s2.end()));
 
     if (len1 < len2) return levenshtein_mbleven2018(s2, s1, max);
 
@@ -179,15 +175,16 @@ int64_t levenshtein_mbleven2018(Range<InputIt1> s1, Range<InputIt2> s2, int64_t
     int64_t dist = max + 1;
 
     for (uint8_t ops : possible_ops) {
-        ptrdiff_t s1_pos = 0;
-        ptrdiff_t s2_pos = 0;
+        auto iter_s1 = s1.begin();
+        auto iter_s2 = s2.begin();
         int64_t cur_dist = 0;
-        while (s1_pos < len1 && s2_pos < len2) {
-            if (s1[s1_pos] != s2[s2_pos]) {
+
+        while (iter_s1 != s1.end() && iter_s2 != s2.end()) {
+            if (*iter_s1 != *iter_s2) {
                 cur_dist++;
                 if (!ops) break;
-                if (ops & 1) s1_pos++;
-                if (ops & 2) s2_pos++;
+                if (ops & 1) iter_s1++;
+                if (ops & 2) iter_s2++;
 #if defined(__GNUC__) && !defined(__clang__) && !defined(__ICC) && __GNUC__ < 10
 #    pragma GCC diagnostic push
 #    pragma GCC diagnostic ignored "-Wconversion"
@@ -198,11 +195,11 @@ int64_t levenshtein_mbleven2018(Range<InputIt1> s1, Range<InputIt2> s2, int64_t
 #endif
             }
             else {
-                s1_pos++;
-                s2_pos++;
+                iter_s1++;
+                iter_s2++;
             }
         }
-        cur_dist += (len1 - s1_pos) + (len2 - s2_pos);
+        cur_dist += std::distance(iter_s1, s1.end()) + std::distance(iter_s2, s2.end());
         dist = std::min(dist, cur_dist);
     }
 
@@ -232,6 +229,8 @@ auto levenshtein_hyrroe2003(const PM_Vec& PM, Range<InputIt1> s1, Range<InputIt2
                             int64_t max = std::numeric_limits<int64_t>::max())
     -> LevenshteinResult<RecordMatrix, RecordBitRow>
 {
+    assert(s1.size() != 0);
+
     /* VP is set to 1^m. Shifting by bitwidth would be undefined behavior */
     uint64_t VP = ~UINT64_C(0);
     uint64_t VN = 0;
@@ -247,9 +246,10 @@ auto levenshtein_hyrroe2003(const PM_Vec& PM, Range<InputIt1> s1, Range<InputIt2
     uint64_t mask = UINT64_C(1) << (s1.size() - 1);
 
     /* Searching */
-    for (ptrdiff_t i = 0; i < s2.size(); ++i) {
+    auto iter_s2 = s2.begin();
+    for (ptrdiff_t i = 0; iter_s2 != s2.end(); ++iter_s2, ++i) {
         /* Step 1: Computing D0 */
-        uint64_t PM_j = PM.get(0, s2[i]);
+        uint64_t PM_j = PM.get(0, *iter_s2);
         uint64_t X = PM_j;
         uint64_t D0 = (((X & VP) + VP) ^ VP) | X | VN;
 
@@ -286,11 +286,104 @@ auto levenshtein_hyrroe2003(const PM_Vec& PM, Range<InputIt1> s1, Range<InputIt2
     return res;
 }
 
+#ifdef RAPIDFUZZ_SIMD
+template <typename VecType, typename InputIt, int _lto_hack = RAPIDFUZZ_LTO_HACK>
+void levenshtein_hyrroe2003_simd(Range<int64_t*> scores, const detail::BlockPatternMatchVector& block,
+                                 const std::vector<size_t>& s1_lengths, Range<InputIt> s2,
+                                 int64_t score_cutoff) noexcept
+{
+#    ifdef RAPIDFUZZ_AVX2
+    using namespace simd_avx2;
+#    else
+    using namespace simd_sse2;
+#    endif
+    static constexpr size_t vec_width = native_simd<VecType>::size();
+    static constexpr size_t vecs = static_cast<size_t>(native_simd<uint64_t>::size());
+    assert(block.size() % vecs == 0);
+
+    native_simd<VecType> zero(VecType(0));
+    native_simd<VecType> one(1);
+    size_t result_index = 0;
+
+    for (size_t cur_vec = 0; cur_vec < block.size(); cur_vec += vecs) {
+        /* VP is set to 1^m */
+        native_simd<VecType> VP(static_cast<VecType>(-1));
+        native_simd<VecType> VN(VecType(0));
+
+        alignas(32) std::array<VecType, vec_width> currDist_;
+        unroll<int, vec_width>(
+            [&](auto i) { currDist_[i] = static_cast<VecType>(s1_lengths[result_index + i]); });
+        native_simd<VecType> currDist(reinterpret_cast<uint64_t*>(currDist_.data()));
+        /* mask used when computing D[m,j] in the paper 10^(m-1) */
+        alignas(32) std::array<VecType, vec_width> mask_;
+        unroll<int, vec_width>([&](auto i) {
+            if (s1_lengths[result_index + i] == 0)
+                mask_[i] = 0;
+            else
+                mask_[i] = static_cast<VecType>(UINT64_C(1) << (s1_lengths[result_index + i] - 1));
+        });
+        native_simd<VecType> mask(reinterpret_cast<uint64_t*>(mask_.data()));
+
+        for (const auto& ch : s2) {
+            /* Step 1: Computing D0 */
+            alignas(32) std::array<uint64_t, vecs> stored;
+            unroll<int, vecs>([&](auto i) { stored[i] = block.get(cur_vec + i, ch); });
+
+            native_simd<VecType> X(stored.data());
+            auto D0 = (((X & VP) + VP) ^ VP) | X | VN;
+
+            /* Step 2: Computing HP and HN */
+            auto HP = VN | ~(D0 | VP);
+            auto HN = D0 & VP;
+
+            /* Step 3: Computing the value D[m,j] */
+            currDist += andnot(one, (HP & mask) == zero);
+            currDist -= andnot(one, (HN & mask) == zero);
+
+            /* Step 4: Computing Vp and VN */
+            HP = (HP << 1) | one;
+            HN = (HN << 1);
+
+            VP = HN | ~(D0 | HP);
+            VN = HP & D0;
+        }
+
+        alignas(32) std::array<VecType, vec_width> distances;
+        currDist.store(distances.data());
+
+        unroll<int, vec_width>([&](auto i) {
+            int64_t score = 0;
+            /* strings of length 0 are not handled correctly */
+            if (s1_lengths[result_index] == 0) {
+                score = s2.size();
+            }
+            /* calculate score under consideration of wraparounds in parallel counter */
+            else {
+                if constexpr (!std::is_same_v<VecType, uint64_t>) {
+                    ptrdiff_t min_dist =
+                        std::abs(static_cast<ptrdiff_t>(s1_lengths[result_index]) - s2.size());
+                    int64_t wraparound_score = static_cast<int64_t>(std::numeric_limits<VecType>::max()) + 1;
+
+                    score = (min_dist / wraparound_score) * wraparound_score;
+                    VecType remainder = static_cast<VecType>(min_dist % wraparound_score);
+
+                    if (distances[i] < remainder) score += wraparound_score;
+                }
+
+                score += static_cast<int64_t>(distances[i]);
+            }
+            scores[static_cast<int64_t>(result_index)] = (score <= score_cutoff) ? score : score_cutoff + 1;
+            result_index++;
+        });
+    }
+}
+#endif
+
 template <typename InputIt1, typename InputIt2>
 int64_t levenshtein_hyrroe2003_small_band(const BlockPatternMatchVector& PM, Range<InputIt1> s1,
                                           Range<InputIt2> s2, int64_t max)
 {
-    /* VP is set to 1^m. Shifting by bitwidth would be undefined behavior */
+    /* VP is set to 1^m. */
     uint64_t VP = ~UINT64_C(0) << (64 - max - 1);
     uint64_t VN = 0;
 
@@ -378,6 +471,7 @@ auto levenshtein_hyrroe2003_small_band(Range<InputIt1> s1, Range<InputIt2> s2, i
 {
     assert(max <= s1.size());
     assert(max <= s2.size());
+    assert(s2.size() >= s1.size() - max);
 
     /* VP is set to 1^m. Shifting by bitwidth would be undefined behavior */
     uint64_t VP = ~UINT64_C(0) << (64 - max - 1);
@@ -403,25 +497,27 @@ auto levenshtein_hyrroe2003_small_band(Range<InputIt1> s1, Range<InputIt2> s2, i
     int64_t break_score = max + s2.size() - (s1.size() - max);
     HybridGrowingHashmap<typename Range<InputIt1>::value_type, std::pair<ptrdiff_t, uint64_t>> PM;
 
-    for (ptrdiff_t j = -max; j < 0; ++j) {
-        auto& x = PM[s1[j + max]];
+    auto iter_s1 = s1.begin();
+    for (ptrdiff_t j = -max; j < 0; ++iter_s1, ++j) {
+        auto& x = PM[*iter_s1];
         x.second = shr64(x.second, j - x.first) | (UINT64_C(1) << 63);
         x.first = j;
     }
 
     /* Searching */
     ptrdiff_t i = 0;
-    for (; i < s1.size() - max; ++i) {
+    auto iter_s2 = s2.begin();
+    for (; i < s1.size() - max; ++iter_s2, ++iter_s1, ++i) {
         /* Step 1: Computing D0 */
         /* update bitmasks online */
         uint64_t PM_j = 0;
-        if (i + max < s1.size()) {
-            auto& x = PM[s1[i + max]];
+        {
+            auto& x = PM[*iter_s1];
             x.second = shr64(x.second, i - x.first) | (UINT64_C(1) << 63);
             x.first = i;
         }
         {
-            auto x = PM.get(s2[i]);
+            auto x = PM.get(*iter_s2);
             PM_j = shr64(x.second, i - x.first);
         }
 
@@ -450,17 +546,18 @@ auto levenshtein_hyrroe2003_small_band(Range<InputIt1> s1, Range<InputIt2> s2, i
         }
     }
 
-    for (; i < s2.size(); ++i) {
+    for (; i < s2.size(); ++iter_s2, ++i) {
         /* Step 1: Computing D0 */
         /* update bitmasks online */
         uint64_t PM_j = 0;
-        if (i + max < s1.size()) {
-            auto& x = PM[s1[i + max]];
+        if (iter_s1 != s1.end()) {
+            auto& x = PM[*iter_s1];
             x.second = shr64(x.second, i - x.first) | (UINT64_C(1) << 63);
             x.first = i;
+            ++iter_s1;
         }
         {
-            auto x = PM.get(s2[i]);
+            auto x = PM.get(*iter_s2);
             PM_j = shr64(x.second, i - x.first);
         }
 
@@ -540,7 +637,8 @@ auto levenshtein_hyrroe2003_block(const BlockPatternMatchVector& PM, Range<Input
         1;
 
     /* Searching */
-    for (ptrdiff_t row = 0; row < s2.size(); ++row) {
+    auto iter_s2 = s2.begin();
+    for (ptrdiff_t row = 0; row < s2.size(); ++iter_s2, ++row) {
         uint64_t HP_carry = 1;
         uint64_t HN_carry = 0;
 
@@ -551,7 +649,7 @@ auto levenshtein_hyrroe2003_block(const BlockPatternMatchVector& PM, Range<Input
 
         auto advance_block = [&](size_t word) {
             /* Step 1: Computing D0 */
-            uint64_t PM_j = PM.get(word, s2[row]);
+            uint64_t PM_j = PM.get(word, *iter_s2);
             uint64_t VN = vecs[word].VN;
             uint64_t VP = vecs[word].VP;
 
@@ -695,76 +793,104 @@ auto levenshtein_hyrroe2003_block(const BlockPatternMatchVector& PM, Range<Input
 
 template <typename InputIt1, typename InputIt2>
 int64_t uniform_levenshtein_distance(const BlockPatternMatchVector& block, Range<InputIt1> s1,
-                                     Range<InputIt2> s2, int64_t max)
+                                     Range<InputIt2> s2, int64_t score_cutoff, int64_t score_hint)
 {
     /* upper bound */
-    max = std::min(max, std::max<int64_t>(s1.size(), s2.size()));
+    score_cutoff = std::min(score_cutoff, std::max<int64_t>(s1.size(), s2.size()));
+    if (score_hint < 31) score_hint = 31;
 
     // when no differences are allowed a direct comparision is sufficient
-    if (max == 0) return !std::equal(s1.begin(), s1.end(), s2.begin(), s2.end());
+    if (score_cutoff == 0) return !std::equal(s1.begin(), s1.end(), s2.begin(), s2.end());
 
-    if (max < std::abs(s1.size() - s2.size())) return max + 1;
+    if (score_cutoff < std::abs(s1.size() - s2.size())) return score_cutoff + 1;
 
     // important to catch, since this causes block to be empty -> raises exception on access
-    if (s1.empty()) {
-        return (s2.size() <= max) ? s2.size() : max + 1;
-    }
+    if (s1.empty()) return (s2.size() <= score_cutoff) ? s2.size() : score_cutoff + 1;
 
     /* do this first, since we can not remove any affix in encoded form
      * todo actually we could at least remove the common prefix and just shift the band
      */
-    if (max >= 4) {
+    if (score_cutoff >= 4) {
         // todo could safe up to 25% even without max when ignoring irrelevant paths
         // in the upper and lower corner
-        int64_t full_band = std::min<int64_t>(s1.size(), 2 * max + 1);
+        int64_t full_band = std::min<int64_t>(s1.size(), 2 * score_cutoff + 1);
 
         if (s1.size() < 65)
-            return levenshtein_hyrroe2003<false, false>(block, s1, s2, max).dist;
+            return levenshtein_hyrroe2003<false, false>(block, s1, s2, score_cutoff).dist;
         else if (full_band <= 64)
-            return levenshtein_hyrroe2003_small_band(block, s1, s2, max);
-        else
-            return levenshtein_hyrroe2003_block<false, false>(block, s1, s2, max).dist;
+            return levenshtein_hyrroe2003_small_band(block, s1, s2, score_cutoff);
+
+        while (score_hint < score_cutoff) {
+            full_band = std::min<int64_t>(s1.size(), 2 * score_hint + 1);
+
+            int64_t score;
+            if (full_band <= 64)
+                score = levenshtein_hyrroe2003_small_band(block, s1, s2, score_hint);
+            else
+                score = levenshtein_hyrroe2003_block<false, false>(block, s1, s2, score_hint).dist;
+
+            if (score <= score_hint) return score;
+
+            if (std::numeric_limits<int64_t>::max() / 2 < score_hint) break;
+
+            score_hint *= 2;
+        }
+
+        return levenshtein_hyrroe2003_block<false, false>(block, s1, s2, score_cutoff).dist;
     }
 
     /* common affix does not effect Levenshtein distance */
     remove_common_affix(s1, s2);
     if (s1.empty() || s2.empty()) return s1.size() + s2.size();
 
-    return levenshtein_mbleven2018(s1, s2, max);
+    return levenshtein_mbleven2018(s1, s2, score_cutoff);
 }
 
 template <typename InputIt1, typename InputIt2>
-int64_t uniform_levenshtein_distance(Range<InputIt1> s1, Range<InputIt2> s2, int64_t max)
+int64_t uniform_levenshtein_distance(Range<InputIt1> s1, Range<InputIt2> s2, int64_t score_cutoff,
+                                     int64_t score_hint)
 {
     /* Swapping the strings so the second string is shorter */
-    if (s1.size() < s2.size()) return uniform_levenshtein_distance(s2, s1, max);
+    if (s1.size() < s2.size()) return uniform_levenshtein_distance(s2, s1, score_cutoff, score_hint);
 
     /* upper bound */
-    max = std::min(max, std::max<int64_t>(s1.size(), s2.size()));
+    score_cutoff = std::min(score_cutoff, std::max<int64_t>(s1.size(), s2.size()));
+    if (score_hint < 31) score_hint = 31;
 
     // when no differences are allowed a direct comparision is sufficient
-    if (max == 0) return !std::equal(s1.begin(), s1.end(), s2.begin(), s2.end());
+    if (score_cutoff == 0) return !std::equal(s1.begin(), s1.end(), s2.begin(), s2.end());
 
     // at least length difference insertions/deletions required
-    if (max < (s1.size() - s2.size())) return max + 1;
+    if (score_cutoff < (s1.size() - s2.size())) return score_cutoff + 1;
 
     /* common affix does not effect Levenshtein distance */
     remove_common_affix(s1, s2);
     if (s1.empty() || s2.empty()) return s1.size() + s2.size();
 
-    if (max < 4) return levenshtein_mbleven2018(s1, s2, max);
+    if (score_cutoff < 4) return levenshtein_mbleven2018(s1, s2, score_cutoff);
 
-    // todo could safe up to 25% even without max when ignoring irrelevant paths
+    // todo could safe up to 25% even without score_cutoff when ignoring irrelevant paths
     // in the upper and lower corner
-    int64_t full_band = std::min<int64_t>(s1.size(), 2 * max + 1);
+    int64_t full_band = std::min<int64_t>(s1.size(), 2 * score_cutoff + 1);
 
     /* when the short strings has less then 65 elements Hyyrös' algorithm can be used */
     if (s2.size() < 65)
-        return levenshtein_hyrroe2003<false, false>(PatternMatchVector(s2), s2, s1, max).dist;
+        return levenshtein_hyrroe2003<false, false>(PatternMatchVector(s2), s2, s1, score_cutoff).dist;
     else if (full_band <= 64)
-        return levenshtein_hyrroe2003_small_band<false>(s1, s2, max).dist;
-    else
-        return levenshtein_hyrroe2003_block<false, false>(BlockPatternMatchVector(s1), s1, s2, max).dist;
+        return levenshtein_hyrroe2003_small_band<false>(s1, s2, score_cutoff).dist;
+    else {
+        BlockPatternMatchVector PM(s1);
+        while (score_hint < score_cutoff) {
+            int64_t score = levenshtein_hyrroe2003_block<false, false>(PM, s1, s2, score_hint).dist;
+            if (score <= score_hint) return score;
+
+            if (std::numeric_limits<int64_t>::max() / 2 < score_hint) break;
+
+            score_hint *= 2;
+        }
+
+        return levenshtein_hyrroe2003_block<false, false>(PM, s1, s2, score_cutoff).dist;
+    }
 }
 
 /**
@@ -843,19 +969,16 @@ void levenshtein_align(Editops& editops, Range<InputIt1> s1, Range<InputIt2> s2,
     int64_t full_band = std::min<int64_t>(s1.size(), 2 * max + 1);
 
     LevenshteinResult<true, false> matrix;
-    do {
-        if (s1.empty() || s2.empty())
-            matrix.dist = s1.size() + s2.size();
-        else if (s1.size() <= 64)
-            matrix = levenshtein_hyrroe2003<true, false>(PatternMatchVector(s1), s1, s2);
-        else if (full_band <= 64)
-            matrix = levenshtein_hyrroe2003_small_band<true>(s1, s2, max);
-        else
-            matrix = levenshtein_hyrroe2003_block<true, false>(BlockPatternMatchVector(s1), s1, s2, max);
-
-        max *= 2;
-    } while (matrix.dist > max);
+    if (s1.empty() || s2.empty())
+        matrix.dist = s1.size() + s2.size();
+    else if (s1.size() <= 64)
+        matrix = levenshtein_hyrroe2003<true, false>(PatternMatchVector(s1), s1, s2);
+    else if (full_band <= 64)
+        matrix = levenshtein_hyrroe2003_small_band<true>(s1, s2, max);
+    else
+        matrix = levenshtein_hyrroe2003_block<true, false>(BlockPatternMatchVector(s1), s1, s2, max);
 
+    assert(matrix.dist <= max);
     if (matrix.dist != 0) {
         if (editops.size() == 0) editops.resize(static_cast<size_t>(matrix.dist));
 
@@ -873,7 +996,8 @@ LevenshteinResult<false, true> levenshtein_row(Range<InputIt1> s1, Range<InputIt
 template <typename InputIt1, typename InputIt2>
 int64_t levenshtein_distance(Range<InputIt1> s1, Range<InputIt2> s2,
                              LevenshteinWeightTable weights = {1, 1, 1},
-                             int64_t max = std::numeric_limits<int64_t>::max())
+                             int64_t score_cutoff = std::numeric_limits<int64_t>::max(),
+                             int64_t score_hint = std::numeric_limits<int64_t>::max())
 {
     if (weights.insert_cost == weights.delete_cost) {
         /* when insertions + deletions operations are free there can not be any edit distance */
@@ -881,26 +1005,27 @@ int64_t levenshtein_distance(Range<InputIt1> s1, Range<InputIt2> s2,
 
         /* uniform Levenshtein multiplied with the common factor */
         if (weights.insert_cost == weights.replace_cost) {
-            // max can make use of the common divisor of the three weights
-            int64_t new_max = ceil_div(max, weights.insert_cost);
-            int64_t distance = uniform_levenshtein_distance(s1, s2, new_max);
+            // score_cutoff can make use of the common divisor of the three weights
+            int64_t new_score_cutoff = ceil_div(score_cutoff, weights.insert_cost);
+            int64_t new_score_hint = ceil_div(score_hint, weights.insert_cost);
+            int64_t distance = uniform_levenshtein_distance(s1, s2, new_score_cutoff, new_score_hint);
             distance *= weights.insert_cost;
-            return (distance <= max) ? distance : max + 1;
+            return (distance <= score_cutoff) ? distance : score_cutoff + 1;
         }
         /*
          * when replace_cost >= insert_cost + delete_cost no substitutions are performed
          * therefore this can be implemented as InDel distance multiplied with the common factor
          */
         else if (weights.replace_cost >= weights.insert_cost + weights.delete_cost) {
-            // max can make use of the common divisor of the three weights
-            int64_t new_max = ceil_div(max, weights.insert_cost);
-            int64_t distance = rapidfuzz::indel_distance(s1, s2, new_max);
+            // score_cutoff can make use of the common divisor of the three weights
+            int64_t new_score_cutoff = ceil_div(score_cutoff, weights.insert_cost);
+            int64_t distance = rapidfuzz::indel_distance(s1, s2, new_score_cutoff);
             distance *= weights.insert_cost;
-            return (distance <= max) ? distance : max + 1;
+            return (distance <= score_cutoff) ? distance : score_cutoff + 1;
         }
     }
 
-    return generalized_levenshtein_wagner_fischer(s1, s2, weights, max);
+    return generalized_levenshtein_wagner_fischer(s1, s2, weights, score_cutoff);
 }
 struct HirschbergPos {
     int64_t left_score;
@@ -1016,34 +1141,47 @@ void levenshtein_align_hirschberg(Editops& editops, Range<InputIt1> s1, Range<In
     }
 }
 
+class Levenshtein : public DistanceBase<Levenshtein, int64_t, 0, std::numeric_limits<int64_t>::max(),
+                                        LevenshteinWeightTable> {
+    friend DistanceBase<Levenshtein, int64_t, 0, std::numeric_limits<int64_t>::max(), LevenshteinWeightTable>;
+    friend NormalizedMetricBase<Levenshtein, LevenshteinWeightTable>;
+
+    template <typename InputIt1, typename InputIt2>
+    static int64_t maximum(Range<InputIt1> s1, Range<InputIt2> s2, LevenshteinWeightTable weights)
+    {
+        return levenshtein_maximum(s1.size(), s2.size(), weights);
+    }
+
+    template <typename InputIt1, typename InputIt2>
+    static int64_t _distance(Range<InputIt1> s1, Range<InputIt2> s2, LevenshteinWeightTable weights,
+                             int64_t score_cutoff, int64_t score_hint)
+    {
+        return levenshtein_distance(s1, s2, weights, score_cutoff, score_hint);
+    }
+};
+
 template <typename InputIt1, typename InputIt2>
 Editops levenshtein_editops(Range<InputIt1> s1, Range<InputIt2> s2, int64_t score_hint)
 {
     Editops editops;
     if (score_hint < 31) score_hint = 31;
 
-    levenshtein_align_hirschberg(editops, s1, s2, 0, 0, 0, score_hint);
+    int64_t score_cutoff = std::max(s1.size(), s2.size());
+    /* score_hint currently leads to calculating the levenshtein distance twice
+     * 1) to find the real distance
+     * 2) to find the alignment
+     * this is only worth it when at least 50% of the runtime could be saved
+     * todo: maybe there is a way to join these two calculations in the future
+     * so it is worth it in more cases
+     */
+    if (std::numeric_limits<int64_t>::max() / 2 > score_hint && 2 * score_hint < score_cutoff)
+        score_cutoff = Levenshtein::distance(s1, s2, {1, 1, 1}, score_cutoff, score_hint);
+
+    levenshtein_align_hirschberg(editops, s1, s2, 0, 0, 0, score_cutoff);
+
     editops.set_src_len(static_cast<size_t>(s1.size()));
     editops.set_dest_len(static_cast<size_t>(s2.size()));
     return editops;
 }
 
-class Levenshtein : public DistanceBase<Levenshtein, LevenshteinWeightTable> {
-    friend DistanceBase<Levenshtein, LevenshteinWeightTable>;
-    friend NormalizedMetricBase<Levenshtein, LevenshteinWeightTable>;
-
-    template <typename InputIt1, typename InputIt2>
-    static int64_t maximum(Range<InputIt1> s1, Range<InputIt2> s2, LevenshteinWeightTable weights)
-    {
-        return levenshtein_maximum(s1, s2, weights);
-    }
-
-    template <typename InputIt1, typename InputIt2>
-    static int64_t _distance(Range<InputIt1> s1, Range<InputIt2> s2, LevenshteinWeightTable weights,
-                             int64_t score_cutoff)
-    {
-        return levenshtein_distance(s1, s2, weights, score_cutoff);
-    }
-};
-
 } // namespace rapidfuzz::detail
